# Text2SQL-LTM - Commercial License & Advanced Protection
# Copyright (c) 2024 Dr. Alban Maxhuni. All rights reserved.
# Contact: info@albanmaxhuni.com
# 
# LEGAL NOTICE:
# This software is protected by commercial license and obfuscation.
# Unauthorized reverse engineering, decompilation, or redistribution is prohibited.
# Commercial license required for production use.
# For licensing inquiries: info@albanmaxhuni.com
#
# TECHNICAL PROTECTION: String obfuscation, license enforcement

import base64
"""
Configuration classes for the TEXT2SQL-LTM system.

This module provides comprehensive configuration management for all aspects
of the Text2SQL agent with long-term memory capabilities.
"""

from typing import Dict, List, Optional, Union, Any
from enum import Enum
from pathlib import Path
import os

from pydantic import BaseModel, Field, validator, root_validator
from pydantic_settings import BaseSettings


class LLMProvider(str, Enum):
    """Supported LLM providers."""
    OPENAI = "openai"
    ANTHROPIC = "anthropic"
    AZURE = "azure"
    HUGGINGFACE = "huggingface"
    LOCAL = "local"


class MemoryBackend(str, Enum):
    """Supported memory storage backends."""
    MEM0 = "mem0"
    REDIS = "redis"
    POSTGRESQL = "postgresql"
    MONGODB = "mongodb"
    SQLITE = "sqlite"
    MEMORY = "memory"  # In-memory for testing


class PrivacyMode(str, Enum):
    """Privacy protection levels."""
    STRICT = "strict"      # Maximum privacy, minimal data retention
    BALANCED = "balanced"  # Balance between privacy and functionality
    PERMISSIVE = "permissive"  # Maximum functionality, longer retention


class MemoryConfig(BaseModel):
    """Configuration for the memory system using mem0.ai."""
    
    # mem0.ai configuration
    mem0_api_key: Optional[str] = Field(None, description="mem0.ai API key")
    mem0_organization_id: Optional[str] = Field(None, description=base64.b64decode("bWVtMC5haSBvcmdhbml6YXRpb24gSUQ=").decode())
    mem0_project_id: Optional[str] = Field(None, description="mem0.ai project ID")
    
    # Storage backend configuration
    storage_backend: MemoryBackend = Field(
        MemoryBackend.MEM0, 
        description=base64.b64decode("UHJpbWFyeSBzdG9yYWdlIGJhY2tlbmQgZm9yIG1lbW9yaWVz").decode()
    )
    storage_url: Optional[str] = Field(None, description=base64.b64decode("U3RvcmFnZSBiYWNrZW5kIGNvbm5lY3Rpb24gVVJM").decode())
    backup_storage_backend: Optional[MemoryBackend] = Field(
        None, 
        description="Backup storage backend"
    )
    
    # User isolation settings
    user_isolation: bool = Field(True, description=base64.b64decode("RW5hYmxlIHVzZXIgbWVtb3J5IGlzb2xhdGlvbg==").decode())
    max_memory_per_user: str = Field("100MB", description=base64.b64decode("TWF4aW11bSBtZW1vcnkgcGVyIHVzZXI=").decode())
    memory_compression: bool = Field(True, description=base64.b64decode("RW5hYmxlIG1lbW9yeSBjb21wcmVzc2lvbg==").decode())
    
    # Memory lifecycle settings
    memory_ttl_days: int = Field(90, description=base64.b64decode("RGVmYXVsdCBtZW1vcnkgVFRMIGluIGRheXM=").decode())
    session_memory_ttl_hours: int = Field(24, description=base64.b64decode("U2Vzc2lvbiBtZW1vcnkgVFRMIGluIGhvdXJz").decode())
    schema_memory_ttl_days: int = Field(180, description=base64.b64decode("U2NoZW1hIG1lbW9yeSBUVEwgaW4gZGF5cw==").decode())
    auto_cleanup: bool = Field(True, description=base64.b64decode("RW5hYmxlIGF1dG9tYXRpYyBtZW1vcnkgY2xlYW51cA==").decode())
    
    # Learning settings
    learning_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIG1lbW9yeSBsZWFybmluZw==").decode())
    feedback_learning: bool = Field(True, description=base64.b64decode("TGVhcm4gZnJvbSB1c2VyIGZlZWRiYWNr").decode())
    pattern_recognition: bool = Field(True, description=base64.b64decode("RW5hYmxlIHF1ZXJ5IHBhdHRlcm4gcmVjb2duaXRpb24=").decode())
    adaptive_personalization: bool = Field(True, description=base64.b64decode("RW5hYmxlIGFkYXB0aXZlIHBlcnNvbmFsaXphdGlvbg==").decode())
    
    # Performance settings
    memory_cache_size: int = Field(1000, description="Memory cache size")
    batch_size: int = Field(100, description=base64.b64decode("QmF0Y2ggc2l6ZSBmb3IgbWVtb3J5IG9wZXJhdGlvbnM=").decode())
    max_concurrent_operations: int = Field(10, description=base64.b64decode("TWF4IGNvbmN1cnJlbnQgbWVtb3J5IG9wZXJhdGlvbnM=").decode())
    
    # Security and privacy
    encryption_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIG1lbW9yeSBlbmNyeXB0aW9u").decode())
    privacy_mode: PrivacyMode = Field(PrivacyMode.BALANCED, description=base64.b64decode("UHJpdmFjeSBwcm90ZWN0aW9uIGxldmVs").decode())
    data_retention_days: int = Field(90, description=base64.b64decode("RGF0YSByZXRlbnRpb24gcGVyaW9kIGluIGRheXM=").decode())
    anonymization_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIGRhdGEgYW5vbnltaXphdGlvbg==").decode())
    
    @validator('storage_url')
    def validate_storage_url(cls, v, values):
        """Validate storage URL format."""
        if v is None:
            return v
        
        backend = values.get('storage_backend')
        if backend == MemoryBackend.REDIS and not v.startswith('redis://'):
            raise ValueError("Redis URL must start with 'redis://'")
        elif backend == MemoryBackend.POSTGRESQL and not v.startswith('postgresql://'):
            raise ValueError("PostgreSQL URL must start with 'postgresql://'")
        elif backend == MemoryBackend.MONGODB and not v.startswith('mongodb://'):
            raise ValueError("MongoDB URL must start with 'mongodb://'")
        
        return v
    
    @validator('max_memory_per_user')
    def validate_memory_size(cls, v):
        """Validate memory size format."""
        if not v.endswith(('B', 'KB', 'MB', 'GB')):
            raise ValueError("Memory size must end with B, KB, MB, or GB")
        return v


class AgentConfig(BaseModel):
    """Configuration for the Text2SQL agent behavior."""
    
    # LLM settings
    llm_provider: LLMProvider = Field(LLMProvider.OPENAI, description="LLM provider")
    llm_model: str = Field("gpt-4", description="LLM model name")
    llm_api_key: Optional[str] = Field(None, description="LLM API key")
    llm_base_url: Optional[str] = Field(None, description="Custom LLM base URL")
    temperature: float = Field(0.1, ge=0.0, le=2.0, description="LLM temperature")
    max_tokens: int = Field(2048, ge=1, le=8192, description=base64.b64decode("TWF4aW11bSB0b2tlbnMgcGVyIHJlc3BvbnNl").decode())
    
    # SQL generation settings
    max_query_complexity: int = Field(100, description=base64.b64decode("TWF4aW11bSBxdWVyeSBjb21wbGV4aXR5IHNjb3Jl").decode())
    enable_query_optimization: bool = Field(True, description=base64.b64decode("RW5hYmxlIHF1ZXJ5IG9wdGltaXphdGlvbg==").decode())
    enable_query_validation: bool = Field(True, description=base64.b64decode("RW5hYmxlIHF1ZXJ5IHZhbGlkYXRpb24=").decode())
    enable_query_explanation: bool = Field(True, description=base64.b64decode("R2VuZXJhdGUgcXVlcnkgZXhwbGFuYXRpb25z").decode())
    
    # Memory integration settings
    memory_influence_weight: float = Field(
        0.3, ge=0.0, le=1.0, 
        description=base64.b64decode("V2VpZ2h0IG9mIG1lbW9yeSBpbmZsdWVuY2Ugb24gcXVlcnkgZ2VuZXJhdGlvbg==").decode()
    )
    context_window_size: int = Field(10, description=base64.b64decode("TnVtYmVyIG9mIHByZXZpb3VzIHF1ZXJpZXMgdG8gY29uc2lkZXI=").decode())
    learning_rate: float = Field(0.01, ge=0.0, le=1.0, description=base64.b64decode("TGVhcm5pbmcgcmF0ZSBmb3IgYWRhcHRhdGlvbg==").decode())
    
    # Performance settings
    query_timeout: int = Field(30, description=base64.b64decode("UXVlcnkgdGltZW91dCBpbiBzZWNvbmRz").decode())
    max_concurrent_sessions: int = Field(100, description=base64.b64decode("TWF4aW11bSBjb25jdXJyZW50IHVzZXIgc2Vzc2lvbnM=").decode())
    cache_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIHF1ZXJ5IHJlc3VsdCBjYWNoaW5n").decode())
    cache_ttl: int = Field(3600, description="Cache TTL in seconds")
    
    # Error handling
    max_retries: int = Field(3, description=base64.b64decode("TWF4aW11bSByZXRyeSBhdHRlbXB0cw==").decode())
    retry_delay: float = Field(1.0, description=base64.b64decode("RGVsYXkgYmV0d2VlbiByZXRyaWVzIGluIHNlY29uZHM=").decode())
    fallback_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIGZhbGxiYWNrIG1lY2hhbmlzbXM=").decode())


class SecurityConfig(BaseModel):
    """Security and privacy configuration."""
    
    # Authentication and authorization
    require_authentication: bool = Field(True, description=base64.b64decode("UmVxdWlyZSB1c2VyIGF1dGhlbnRpY2F0aW9u").decode())
    session_secret_key: str = Field(..., description=base64.b64decode("U2VjcmV0IGtleSBmb3Igc2Vzc2lvbiBtYW5hZ2VtZW50").decode())
    token_expiry_hours: int = Field(24, description=base64.b64decode("VG9rZW4gZXhwaXJ5IHRpbWUgaW4gaG91cnM=").decode())
    
    # Data protection
    encrypt_memories: bool = Field(True, description=base64.b64decode("RW5jcnlwdCBzdG9yZWQgbWVtb3JpZXM=").decode())
    encryption_key: Optional[str] = Field(None, description=base64.b64decode("RW5jcnlwdGlvbiBrZXkgZm9yIG1lbW9yaWVz").decode())
    hash_user_ids: bool = Field(True, description=base64.b64decode("SGFzaCB1c2VyIElEcyBmb3IgcHJpdmFjeQ==").decode())
    
    # Access control
    rate_limiting_enabled: bool = Field(True, description="Enable rate limiting")
    max_requests_per_minute: int = Field(60, description=base64.b64decode("TWF4IHJlcXVlc3RzIHBlciBtaW51dGUgcGVyIHVzZXI=").decode())
    ip_whitelist: List[str] = Field(default_factory=list, description="IP whitelist")
    
    # Audit and compliance
    audit_logging: bool = Field(True, description="Enable audit logging")
    gdpr_compliance: bool = Field(True, description=base64.b64decode("RW5hYmxlIEdEUFIgY29tcGxpYW5jZSBmZWF0dXJlcw==").decode())
    data_export_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIHVzZXIgZGF0YSBleHBvcnQ=").decode())
    data_deletion_enabled: bool = Field(True, description=base64.b64decode("RW5hYmxlIHVzZXIgZGF0YSBkZWxldGlvbg==").decode())
    
    # Security headers and policies
    cors_origins: List[str] = Field(default_factory=list, description="CORS allowed origins")
    content_security_policy: Optional[str] = Field(None, description=base64.b64decode("Q29udGVudCBTZWN1cml0eSBQb2xpY3k=").decode())


class PerformanceConfig(BaseModel):
    """Performance optimization configuration."""
    
    # Connection pooling
    db_pool_size: int = Field(20, description=base64.b64decode("RGF0YWJhc2UgY29ubmVjdGlvbiBwb29sIHNpemU=").decode())
    db_max_overflow: int = Field(30, description=base64.b64decode("RGF0YWJhc2UgY29ubmVjdGlvbiBwb29sIG92ZXJmbG93").decode())
    redis_pool_size: int = Field(50, description=base64.b64decode("UmVkaXMgY29ubmVjdGlvbiBwb29sIHNpemU=").decode())
    
    # Caching
    enable_query_cache: bool = Field(True, description=base64.b64decode("RW5hYmxlIHF1ZXJ5IHJlc3VsdCBjYWNoaW5n").decode())
    enable_schema_cache: bool = Field(True, description=base64.b64decode("RW5hYmxlIHNjaGVtYSBjYWNoaW5n").decode())
    enable_memory_cache: bool = Field(True, description=base64.b64decode("RW5hYmxlIG1lbW9yeSBjYWNoaW5n").decode())
    cache_size_mb: int = Field(256, description="Cache size in MB")
    
    # Async and concurrency
    max_workers: int = Field(10, description=base64.b64decode("TWF4aW11bSB3b3JrZXIgdGhyZWFkcw==").decode())
    async_batch_size: int = Field(100, description=base64.b64decode("QXN5bmMgb3BlcmF0aW9uIGJhdGNoIHNpemU=").decode())
    connection_timeout: int = Field(30, description=base64.b64decode("Q29ubmVjdGlvbiB0aW1lb3V0IGluIHNlY29uZHM=").decode())
    
    # Memory optimization
    memory_limit_mb: int = Field(1024, description="Memory limit in MB")
    gc_threshold: int = Field(1000, description=base64.b64decode("R2FyYmFnZSBjb2xsZWN0aW9uIHRocmVzaG9sZA==").decode())
    enable_memory_profiling: bool = Field(False, description=base64.b64decode("RW5hYmxlIG1lbW9yeSBwcm9maWxpbmc=").decode())
    
    # Monitoring
    enable_metrics: bool = Field(True, description=base64.b64decode("RW5hYmxlIHBlcmZvcm1hbmNlIG1ldHJpY3M=").decode())
    metrics_port: int = Field(9090, description="Metrics server port")
    health_check_interval: int = Field(30, description=base64.b64decode("SGVhbHRoIGNoZWNrIGludGVydmFsIGluIHNlY29uZHM=").decode())


class Settings(BaseSettings):
    """Global application settings loaded from environment variables."""
    
    # Application settings
    app_name: str = Field(base64.b64decode("YWktcHJpc2h0aW5hLVRFWFQyU1FMLUxUTQ==").decode(), description="Application name")
    app_version: str = Field("1.0.0", description="Application version")
    debug: bool = Field(False, description="Debug mode")
    log_level: str = Field("INFO", description="Logging level")
    
    # Server settings
    host: str = Field("0.0.0.0", description="Server host")
    port: int = Field(8000, description="Server port")
    workers: int = Field(1, description=base64.b64decode("TnVtYmVyIG9mIHdvcmtlciBwcm9jZXNzZXM=").decode())
    
    # Configuration file paths
    config_file: Optional[Path] = Field(None, description=base64.b64decode("Q29uZmlndXJhdGlvbiBmaWxlIHBhdGg=").decode())
    secrets_file: Optional[Path] = Field(None, description="Secrets file path")
    
    # Environment-specific settings
    environment: str = Field("development", description="Environment name")
    
    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"
        case_sensitive = False
        
    @root_validator
    def validate_environment(cls, values):
        """Validate environment-specific settings."""
        env = values.get('environment', 'development')
        
        if env == 'production':
            # Production-specific validations
            if values.get('debug', False):
                raise ValueError("Debug mode should be disabled in production")
            if values.get('log_level', 'INFO') == 'DEBUG':
                values['log_level'] = 'INFO'
        
        return values


def load_config(
    config_file: Optional[Union[str, Path]] = None,
    **overrides
) -> Dict[str, Any]:
    """
    Load configuration from file and environment variables.
    
    Args:
        config_file: Path to configuration file
        **overrides: Configuration overrides
        
    Returns:
        Complete configuration dictionary
    """
    # Load base settings
    settings = Settings()
    
    config = {
        "memory": MemoryConfig(),
        "agent": AgentConfig(),
        "security": SecurityConfig(session_secret_key=os.urandom(32).hex()),
        "performance": PerformanceConfig(),
        "settings": settings,
    }
    
    # Load from file if provided
    if config_file:
        config_path = Path(config_file)
        if config_path.exists():
            import yaml
            with open(config_path) as f:
                file_config = yaml.safe_load(f)
            
            # Update config with file values
            for section, values in file_config.items():
                if section in config and isinstance(values, dict):
                    for key, value in values.items():
                        if hasattr(config[section], key):
                            setattr(config[section], key, value)
    
    # Apply overrides
    for key, value in overrides.items():
        if '.' in key:
            section, field = key.split('.', 1)
            if section in config and hasattr(config[section], field):
                setattr(config[section], field, value)
        else:
            if key in config:
                config[key] = value
    
    return config


# Global configuration instance
_config = None

def get_config() -> Dict[str, Any]:
    """Get the global configuration instance."""
    global _config
    if _config is None:
        _config = load_config()
    return _config

def set_config(config: Dict[str, Any]) -> None:
    """Set the global configuration instance."""
    global _config
    _config = config
